﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

//Requires a reference to UnitTestLib

namespace UnitTestInterface
{
    class Runner
    {
        public static void RunTest(string fname,
                                   PrintDelegate append,
                                   PrintDelegate appendLine)
        {
            Assembly a = Assembly.LoadFile(fname);

            foreach (Type t in a.GetTypes())
            {
                if (ContainsTestAttr(t.GetCustomAttributes(true)))
                {
                    appendLine(t.Name + " is marked for testing");
                    foreach (MethodInfo mi in t.GetMethods())
                    {
                        if (ContainsTestAttr(mi.GetCustomAttributes(true)))
                        {
                            object testobj = Instantiate(t);
                            if (mi.ReturnType == typeof(bool) &&
                               mi.GetParameters().Length == 0)
                            {
                                append("  " + mi.Name + ".....");
                                if ((bool)mi.Invoke(testobj, null))
                                {
                                    appendLine("passed");
                                }
                                else
                                {
                                    appendLine("FAILED");
                                }
                            }

                        }
                    }
                    appendLine("");

                }
                else
                {
                    appendLine(t.Name + " is not marked for testing, skipping");
                    appendLine("");
                }

            }
        }

        static bool ContainsTestAttr(Object[] os)
        {
            foreach (object o in os)
            {
                if (o is UnitTestLib.TestAttribute)
                    return true;
            }

            return false;
        }

        static object Instantiate(Type t)
        {
            ConstructorInfo c = t.GetConstructor(new Type[] { });
            if (c == null)
            {
                throw new ArgumentException(
                    "Type must have a nullary constructor");
            }
            return c.Invoke(new object[] { });
        }
    }
}
